# Quality Measures for Continuous-Time Dynamic Graph Generative Models

This repository contains the code used to produce the experiments for *Quality Measures for Continuous-Time Dynamic Graph Generative Models*. Please note that this is an anonymized version of the implementation created for review purposes.

## Setup

Install requirements:

```
pip install -r requirements.txt
```

Please note that installing Pytorch and Pytorch Geometric may require special install procedures depending on platform. Please refer to [Pytorch](https://pytorch.org/get-started/locally/) and [Pytorch Geometric](https://pytorch-geometric.readthedocs.io/en/latest/notes/installation.html) documentation for more details.

## Reproducing Experiments

Reproducing the experiments presented in Section 4 of our main work involves the following high-level steps:

1. Generate perturbed data
2. Calculate function descriptors
3. Calculate distances and correlations
4. Produce visualizations and data for tables

Each step is further detailed in the sections below.

## Generate Perturbed Data

Perturbed data for fidelity experiments:

```
python perturb_fidelity.py
```

This script contains adjustable macros for modifying the output directory to save perturbed datasets, number of seeds to produce datasets across, the number of events in each dataset, and the datasets to perform the experiments for. The Jodie [1] datasets will download automatically upon launching the script. All datasets used are freely available and used according to their respective licenses.

Producing perturbed data for diversity experiments requires first training a Temporal Graph Network (TGN) [2]. To do so:

```
python train_tgn.py
```

This script can be modified to customize the save location of each TGN model.

Next, to produce perturbations for diversity experiments:

```
python perturb_diversity.py
```

This also script contains adjustable macros for customizing the output directory to save perturbed datasets, number of seeds to produce datasets across, the number of events in each dataset, and the datasets to perform the experiments for.

## Calulate Function Descriptors

Calculate descriptors for all metrics, including tested baselines:

```
python function_descriptors.py
```

This script can be customized to change hyperparameters for JL-metric, number of events to calculate descriptors for, number of CPUs to use if using multiprocessing, path of perturbed_data, and path to save descriptors.

## Calculate Distances and Correlations
To calculate distances and Spearman rank correlations for the JL-metric and baselines:

```
python distances.py
```

This script requires a path to function descriptors calculated in the previous step. It can be customized to change output file paths and correlation type (either Pearson or Spearman).

## Produce Visualizations and other data

Correlation distribution plots can be produced by:

```
python correlation_distribution_plots.py
```

This script requires a path to the saved correlation values and can be modified to change output directory for correlation distribution plots, and datasets plotted. The script will additionally print median correlation values for each experiment which correspond to the median values in Table 1 of our main work.

Sample efficiency can be calculated by:

```
python sample_efficiency.py 
```

Similarly, for computational efficiency:

```
python computational_efficiency.py
```



## Random seeding

As described in our main work, all of our experiments are seeded insofar as possible and run multiple times to ensure reproducability. For these experiments all models are run 10 times with random seed starting at 1 for the first experiment and increased by 1 for each subsequent eperiment (1-10).

## References
[1] Kumar, Srijan, Xikun Zhang, and Jure Leskovec. "Predicting dynamic embedding trajectory in temporal interaction networks." Proceedings of the 25th ACM SIGKDD international conference on knowledge discovery & data mining. 2019.

[2] Rossi, Emanuele, et al. "Temporal graph networks for deep learning on dynamic graphs." arXiv preprint arXiv:2006.10637 (2020).
